@@ -15,8 +15,6 @@ You should have received a copy of the GNU General Public License
 along with poliBeePsync. If not, see <http://www.gnu.org/licenses/>.
 """
 
-__version__ = 0.1
-
 from polibeepsync import filesettings
 from requests import ConnectionError, Timeout
 from appdirs import user_config_dir, user_data_dir
@@ -24,27 +22,61 @@ import os
 import pickle
 import sys
 from polibeepsync.common import User, InvalidLoginError
-import PySide
-from PySide.QtCore import *
+from PySide.QtCore import (QThread, QObject, Signal, QAbstractTableModel,
+                           QModelIndex, Qt, Slot, QTimer, QLocale, QPoint)
 from PySide.QtGui import (QApplication, QWidget, QTextCursor,
-                           QMenu, QAction, QIcon,
-                           QIcon, QFileDialog,
-                           QVBoxLayout, QLabel,  QSystemTrayIcon,
-                            qApp, QDialog, QPixmap, QTextEdit,
-                            )
+                          QMenu, QAction, QFileDialog,
+                          QVBoxLayout, QLabel, QSystemTrayIcon,
+                          qApp, QDialog)
 
 from polibeepsync.ui_resizable import Ui_Form
+import re
+import logging
+import sys
+
+LEVELS = {'debug': logging.DEBUG,
+          'info': logging.INFO,
+          'warning': logging.WARNING,
+          'error': logging.ERROR,
+          'critical': logging.CRITICAL,
+}
+
+if len(sys.argv) > 1:
+    level_name = sys.argv[1]
+    level = LEVELS.get(level_name, logging.NOTSET)
+    logging.basicConfig(level=level)
+
+
+def read(*names, **kwargs):
+    with open(
+            os.path.join(os.path.dirname(__file__), *names),
+            encoding=kwargs.get("encoding", "utf8")
+    ) as fp:
+        return fp.read()
+
+
+def find_version(*file_paths):
+    version_file = read(*file_paths)
+    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
+                              version_file, re.M)
+    if version_match:
+        return version_match.group(1)
+    raise RuntimeError("Unable to find version string.")
+
+
+__version__ = find_version("__init__.py")
 
 
 class MySignal(QObject):
     sig = Signal(str)
 
+
 class CoursesSignal(QObject):
     sig = Signal(list)
 
 
 class DownloadThread(QThread):
-    def __init__(self, user, topdir, parent = None):
+    def __init__(self, user, topdir, parent=None):
         QThread.__init__(self, parent)
         self.exiting = False
         self.course_finished = MySignal()
@@ -70,10 +102,12 @@ class DownloadThread(QThread):
                         self.course_finished.sig.emit(text)
                     except Exception as err:
                         self.signal_error.sig.emit(str(err))
+                        logging.critical(str(err))
             self.exiting = True
 
+
 class LoginThread(QThread):
-    def __init__(self, user, parent = None):
+    def __init__(self, user, parent=None):
         QThread.__init__(self, parent)
         self.exiting = False
         self.signal_ok = MySignal()
@@ -81,40 +115,44 @@ class LoginThread(QThread):
         self.user = user
 
     def run(self):
-        while self.exiting==False:
+        while self.exiting == False:
             try:
                 self.user.logout()
                 self.user.login()
                 if self.user.logged == True:
-                    self.exiting=True
+                    self.exiting = True
                     self.signal_ok.sig.emit('Successful login.')
             except IndexError:
-                pass
+                logging.critical(str(err))
                 self.exiting = True
                 self.signal_error.sig.emit('Already logged-in.')
             except InvalidLoginError:
                 self.user.logout()
                 self.exiting = True
                 self.signal_error.sig.emit('Login failed.')
+                logging.critical(str(err))
             except ConnectionError as err:
                 self.user.logout()
                 self.exiting = True
                 self.signal_error.sig.emit('I can\'t connect to the server.'
-                                 ' Is the Internet connection working?')
+                                           ' Is the Internet connection working?')
+                logging.critical(str(err))
             except Timeout as err:
                 self.user.logout()
                 self.exiting = True
                 self.signal_error.sig.emit("The timeout time has been reached."
-                                 " Is the Internet connection working?")
+                                           " Is the Internet connection working?")
+                logging.critical(str(err))
 
             except Exception as err:
                 self.user.logout()
                 self.exiting = True
                 self.signal_error.sig.emit("An error occurred.")
+                logging.critical(str(err))
 
 
 class RefreshCoursesThread(QThread):
-    def __init__(self, user, parent = None):
+    def __init__(self, user, parent=None):
         QThread.__init__(self, parent)
         self.exiting = False
         self.refreshed = MySignal()
@@ -124,15 +162,16 @@ class RefreshCoursesThread(QThread):
         self.user = user
 
     def run(self):
-        while self.exiting==False:
+        while self.exiting == False:
             most_recent = self.user.get_online_courses()
             last = self.user.available_courses
-            new = most_recent -last
+            new = most_recent - last
             removable = last - most_recent
-            if len(removable) >0:
+            if len(removable) > 0:
                 self.refreshed.sig.emit('The following courses have'
                                         ' been removed because they '
-                      'aren\'t available online: {}'.format(removable))
+                                        'aren\'t available online: {}'.format(
+                    removable))
             if len(new) > 0:
                 for course in new:
                     course.save_folder_name = course.simplify_name(course.name)
@@ -148,14 +187,13 @@ class RefreshCoursesThread(QThread):
             # nel main thread chaimare dumpUser()
             # e emettere segnale che passa new e removable
 
-            #for course in new:
-            #    self.courses_model.insertRows(0, 1, course)
+            # for course in new:
+            # self.courses_model.insertRows(0, 1, course)
             #for course in removable:
             #    index = self.courses_model.courses.index(course)
             #    self.courses_model.removeRows(index, 1)
 
 
-
 class CoursesListModel(QAbstractTableModel):
     def __init__(self, courses):
         QAbstractTableModel.__init__(self)
@@ -168,15 +206,15 @@ class CoursesListModel(QAbstractTableModel):
     def columnCount(self, parent=QModelIndex()):
         return 3
 
-    def insertRows(self, position, rows, newcourse, parent= QModelIndex()):
-        self.beginInsertRows(parent, position, position + rows -1)
+    def insertRows(self, position, rows, newcourse, parent=QModelIndex()):
+        self.beginInsertRows(parent, position, position + rows - 1)
         for row in range(rows):
             self.courses.insert(position, newcourse)
         self.endInsertRows()
         return True
 
-    def removeRows(self, position, rows, parent = QModelIndex()):
-        self.beginRemoveRows(parent, position, position + rows -1)
+    def removeRows(self, position, rows, parent=QModelIndex()):
+        self.beginRemoveRows(parent, position, position + rows - 1)
         for row in range(rows):
             del self.courses[position]
         self.endRemoveRows()
@@ -187,10 +225,11 @@ class CoursesListModel(QAbstractTableModel):
             flags = Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable
             return flags
         elif index.column() == 1:
-            flags = Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable\
-            | Qt.ItemIsUserCheckable
+            flags = Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable \
+                    | Qt.ItemIsUserCheckable
             return flags
-        else: return Qt.ItemIsEnabled
+        else:
+            return Qt.ItemIsEnabled
 
     def setData(self, index, value, role=Qt.EditRole):
         if role == Qt.EditRole:
@@ -227,6 +266,7 @@ class CoursesListModel(QAbstractTableModel):
             elif col == 2:
                 return "Save as"
 
+
 class MainWindow(QWidget, Ui_Form):
     def __init__(self, parent=None):
         super(MainWindow, self).__init__(parent)
@@ -243,7 +283,7 @@ class MainWindow(QWidget, Ui_Form):
         self.load_data()
 
         self.timer.timeout.connect(self.syncfiles)
-        self.timer.start(1000*60*int(self.settings['UpdateEvery']))
+        self.timer.start(1000 * 60 * int(self.settings['UpdateEvery']))
 
         self.loginthread = LoginThread(self.user)
         self.loginthread.signal_error.sig.connect(self.myStream_message)
@@ -276,15 +316,13 @@ class MainWindow(QWidget, Ui_Form):
         self.courses_model.dataChanged.connect(self.dumpUser)
         self.syncNow.clicked.connect(self.syncfiles)
 
-
-
         if self.settings['SyncNewCourses'] == str(True):
             self.sync_new = Qt.Checked
         else:
             self.sync_new = Qt.Unchecked
 
-        #if self.settings['NotifyNewCourses'] == str(True):
-        #   self.notify_new = Qt.Checked
+        # if self.settings['NotifyNewCourses'] == str(True):
+        # self.notify_new = Qt.Checked
         #else:
         #    self.notify_new = Qt.Unchecked
 
@@ -311,23 +349,24 @@ class MainWindow(QWidget, Ui_Form):
                 elem.sync = True
 
 
-
     def load_settings(self):
         for path in [user_config_dir(self.appname),
                      user_data_dir(self.appname)]:
             try:
                 os.makedirs(path, exist_ok=True)
             except OSError as err:
+                logging.critical(str(err))
                 if not os.path.isdir(path):
                     self.myStream_message(str(err))
         self.settings_path = os.path.join(user_config_dir(self.appname),
-                                     self.settings_fname)
+                                          self.settings_fname)
         defaults = {
-        'UpdateEvery': '60',
-        'RootFolder': os.path.join(os.path.expanduser('~'), self.appname),
-        'SyncNewCourses': 'False'
+            'UpdateEvery': '60',
+            'RootFolder': os.path.join(os.path.expanduser('~'), self.appname),
+            'SyncNewCourses': 'False'
         }
-        self.settings = filesettings.settingsFromFile(self.settings_path, defaults)
+        self.settings = filesettings.settingsFromFile(self.settings_path,
+                                                      defaults)
 
     def load_data(self):
         try:
@@ -336,21 +375,23 @@ class MainWindow(QWidget, Ui_Form):
                 self.user = pickle.load(f)
                 self.myStream_message("Data has been loaded successfully.")
         except FileNotFoundError as err:
+            logging.critical(str(err))
             self.user = User('', '')
             complete_message = str(err) + " ".join([
-    "\nThis error means that no data can be found in the predefined",
-    "directory. Ignore this if you're using poliBeePsync for "
-    "the first time."])
+                "\nThis error means that no data can be found in the predefined",
+                "directory. Ignore this if you're using poliBeePsync for "
+                "the first time."])
             self.myStream_message(complete_message)
         except Exception as err:
+            logging.critical(str(err))
             self.user = User('', '')
             self.myStream_message(str(err))
 
     def loginstatus(self, status):
         self.login_attempt.setText(status)
 
-    #@Slot(int)
-    #def notifynew(self, state):
+    # @Slot(int)
+    # def notifynew(self, state):
     #    if state == 2:
     #        self.settings['NotifyNewCourses'] = 'True'
     #    else:
@@ -369,7 +410,7 @@ class MainWindow(QWidget, Ui_Form):
     def updateminuteslot(self, minutes):
         self.settings['UpdateEvery'] = str(minutes)
         filesettings.settingsToFile(self.settings, self.settings_path)
-        self.timer.start(1000*60*int(self.settings['UpdateEvery']))
+        self.timer.start(1000 * 60 * int(self.settings['UpdateEvery']))
 
     @Slot(str)
     def rootfolderslot(self, path):
@@ -380,8 +421,9 @@ class MainWindow(QWidget, Ui_Form):
     def chooserootdir(self):
         currentdir = self.settings['RootFolder']
         flags = QFileDialog.DontResolveSymlinks | QFileDialog.ShowDirsOnly
-        newroot =  QFileDialog.getExistingDirectory(None,
-            "Open Directory", currentdir, flags)
+        newroot = QFileDialog.getExistingDirectory(None,
+                                                   "Open Directory",
+                                                   currentdir, flags)
         if newroot != "" and str(newroot) != currentdir:
             self.settings['RootFolder'] = str(newroot)
             filesettings.settingsToFile(self.settings, self.settings_path)
@@ -391,9 +433,10 @@ class MainWindow(QWidget, Ui_Form):
             # if there's a cleaner approach
             del self.downloadthread
             self.downloadthread = DownloadThread(self.user,
-                self.settings['RootFolder'])
+                                                 self.settings['RootFolder'])
             self.downloadthread.dumpuser.sig.connect(self.dumpUser)
-            self.downloadthread.course_finished.sig.connect(self.myStream_message)
+            self.downloadthread.course_finished.sig.connect(
+                self.myStream_message)
             self.downloadthread.signal_error.sig.connect(self.myStream_message)
 
 
@@ -407,6 +450,7 @@ class MainWindow(QWidget, Ui_Form):
                                       .format(newcode))
         except Exception as err:
             self.myStream_message(str(err))
+            logging.critical(str(err))
 
 
     def setpassword(self):
@@ -417,6 +461,7 @@ class MainWindow(QWidget, Ui_Form):
             self.myStream_message("Password changed.")
         except Exception as err:
             self.myStream_message(str(err))
+            logging.critical(str(err))
 
     def testlogin(self):
         if not self.loginthread.isRunning():
@@ -465,7 +510,6 @@ class MainWindow(QWidget, Ui_Form):
             self.downloadthread.start()
 
 
-
     @Slot(str)
     def myStream_message(self, message):
         self.status.moveCursor(QTextCursor.End)
@@ -500,9 +544,13 @@ class MainWindow(QWidget, Ui_Form):
         label.setWordWrap(True)
         label.setObjectName("label")
         verticalLayout.addWidget(label)
-        text = "<html><head/><body><p>poliBeePsync version {}.</p><p>poliBeePsync is a program written by Davide Olianas, released under GNU GPLv3+.</p><p><br/></p><p>More information is available on the <a href=\"http://www.davideolianas.com/polibeepsync\"><span style=\" text-decoration: underline; color:#0000ff;\">official website</span></a>.</p></body></html>".format(__version__)
-        Dialog.setWindowTitle(QApplication.translate("Dialog", "About poliBeePsync", None, QApplication.UnicodeUTF8))
-        label.setText(QApplication.translate("Dialog", text, None, QApplication.UnicodeUTF8))
+        text = "<html><head/><body><p>poliBeePsync version {}.</p><p>poliBeePsync is a program written by Davide Olianas, released under GNU GPLv3+.</p><p><br/></p><p>More information is available on the <a href=\"http://www.davideolianas.com/polibeepsync\"><span style=\" text-decoration: underline; color:#0000ff;\">official website</span></a>.</p></body></html>".format(
+            __version__)
+        Dialog.setWindowTitle(
+            QApplication.translate("Dialog", "About poliBeePsync", None,
+                                   QApplication.UnicodeUTF8))
+        label.setText(QApplication.translate("Dialog", text, None,
+                                             QApplication.UnicodeUTF8))
         Dialog.exec_()
 
 
@@ -512,5 +560,6 @@ def main():
     frame.show()
     sys.exit(app.exec_())
 
+
 if __name__ == '__main__':
     main()
